package at.xaxa.ledger.ui.entry.edit

import android.util.Log
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.ExposedDropdownMenuBox
import androidx.compose.material3.ExposedDropdownMenuDefaults
import androidx.compose.material3.ExposedDropdownMenuDefaults.TrailingIcon
import androidx.compose.material3.Icon
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.runtime.LaunchedEffect
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import at.xaxa.ledger.data.Entry
import at.xaxa.ledger.ui.AppViewModelProvider
import at.xaxa.ledger.ui.ButtonDanger
import at.xaxa.ledger.ui.ButtonSuccess
import at.xaxa.ledger.ui.DatePickerDocked
import at.xaxa.ledger.ui.category.iconNames
import at.xaxa.ledger.ui.category.icons

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun Edit(
    modifier: Modifier = Modifier,
    onCardClick: () -> Unit,
    editViewModel: EditViewModel = viewModel(factory = AppViewModelProvider.Factory),
    onValueChange: (Entry) -> Unit = {}
) {
    val entry = editViewModel.editUiState.entry
    val category = editViewModel.categoryUi.category
    val categories by editViewModel.categoryListUiState.categories.collectAsState(initial = emptyList())

    var selectedIconIndex by remember { mutableStateOf(category.icon) }
    var selectedCategoryName by remember { mutableStateOf(category.categoryName) }
    var selectedCategory by remember { mutableIntStateOf(category._id) }

    var expanded by remember { mutableStateOf(false) }

    LaunchedEffect(category) {
        selectedIconIndex = category.icon
        selectedCategoryName = category.categoryName
        selectedCategory = category._id
    }


    Column(
        modifier = modifier
            .fillMaxSize()
            .padding(16.dp, 0.dp),
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Column(
            Modifier.weight(1f)
        ) {
            OutlinedTextField(
                value = entry.name,
                onValueChange = { editViewModel.updateEntry(entry.copy(name = it)) },
                label = { Text("Name") },
                modifier = Modifier.fillMaxWidth()
            )
            OutlinedTextField(
                value = entry.amount.toString(),
                onValueChange = {
                    val isValidSpending = it.matches(Regex("^[+-]?\\d*(\\.\\d{0,2})?$"))
                    if (isValidSpending) {
                        editViewModel.updateEntry(entry.copy(amount = it.toFloat()))
                    }
                },
                label = { Text("Amount") },
                modifier = Modifier.fillMaxWidth()
            )

            ExposedDropdownMenuBox(
                expanded = expanded,
                onExpandedChange = { expanded = it }
            ) {
                OutlinedTextField(
                    value = selectedCategoryName, // Show selected category name
                    onValueChange = {},
                    label = { Text("Category") },
                    readOnly = true,
                    leadingIcon = {
                        Icon(
                            imageVector = icons[selectedIconIndex], // Show selected icon
                            contentDescription = "Leading Icon"
                        )
                    },
                    trailingIcon = {
                        TrailingIcon(expanded = expanded)
                    },
                    modifier = Modifier
                        .menuAnchor()
                        .fillMaxWidth()
                )

                ExposedDropdownMenu(
                    expanded = expanded,
                    onDismissRequest = { expanded = false }
                ) {
                    categories.forEachIndexed { index, category ->
                        DropdownMenuItem(
                            text = { Text(text = category.categoryName) },
                            onClick = {
                                selectedIconIndex = category.icon // Update selected icon index
                                selectedCategoryName = category.categoryName // Update selected category name
                                selectedCategory = category._id // Update selected category ID
                                expanded = false
                            },
                            leadingIcon = {
                                Icon(
                                    imageVector = icons[category.icon],
                                    contentDescription = iconNames[category.icon]
                                )
                            }
                        )
                    }
                }
            }

            DatePickerDocked(entry) { dateMilis ->
                editViewModel.updateEntry(entry.copy(date = dateMilis))
            }
        }

        Column(
            modifier = Modifier.fillMaxWidth(),
            horizontalAlignment = Alignment.CenterHorizontally
        ) {
            ButtonDanger(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(bottom = 8.dp),
                text = "Delete",
                onClick = {
                    editViewModel.deleteEntry()
                    onCardClick()
                }
            )
            ButtonSuccess(
                modifier = Modifier.fillMaxWidth(),
                text = "Done",
                onClick = {
                    editViewModel.saveEntry()
                    onCardClick()
                }
            )
        }
    }
}